-- Daily Tasks window handler
-- coding: utf-8 'ä

import "Turbine.UI.Lotro"
import "Vinny.Common.DropMenu"
import "Vinny.Common.ToolTip"

local Any,All = "<Any rep>","<All Locations>"
local DT_Filters = { ["All"]="dt", ["Usable"]="dtu", ["Unusable"]="dtx", ["Complete"]="dtc", ["Junk"]="dtj" }
local Zone_list = DT_Sort(DT_Data.zone)
local Player_list = DT_Sort(DT_Settings.player)
local Bag_list = {"1","2","3","4","5","6"}
local labelFont = Turbine.UI.Lotro.Font.TrajanPro14
local foreColor = Turbine.UI.Color( 0.9, 0.9, 0 )
local backColor = Turbine.UI.Color( 0, 0, 0.1 )
local Button = Turbine.UI.Lotro.Button
local Label = Turbine.UI.Lotro.GoldButton
local TextBox = Turbine.UI.TextBox
local Left = Turbine.UI.ContentAlignment.MiddleLeft
local Center = Turbine.UI.ContentAlignment.MiddleCenter
local Right = Turbine.UI.ContentAlignment.MiddleRight
local CheckBox = Turbine.UI.Lotro.CheckBox
local DropMenu = Vinny.Common.DropMenu
local Slot = Turbine.UI.Lotro.Quickslot
local Alias = Turbine.UI.Lotro.ShortcutType.Alias
local Qsize = 36

DT_Window = class( Turbine.UI.Lotro.Window )

local function distance(dy,dx) return math.sqrt(dy*dy+dx*dx) end

local function locV(str,neg)
    local nbr = tonumber(str:sub(1,-2))
    if neg:find(str:sub(-1)) then nbr = -nbr end
    return nbr
end

function DT_Window:AddField(control, text, pos, size)
	local field = control()
	field:SetParent( self )
	if text then field:SetText( text ) end
	field:SetPosition( pos.x,pos.y )
	field:SetSize( size.x,size.y )
	if control==Button or control==DropMenu or not text then return field end
	field:SetFont( labelFont )
	field:SetForeColor( foreColor )
	field:SetBackColor( backColor )
	field:SetTextAlignment( Center )
	return field
end

function DT_Click(Item,button)
	Item.Click = function( sender,args )
		local selected = sender:GetText()
		button:SetText(selected)
		print( "Selected "..selected )
		if button.Action then
			button:Action(selected)
		end
		if button==DT_window.locationButton then
			if selected~=All then
				DT_window.Homestead:SetChecked(false)
				Homestead = false
			end
		end
	end
end

function DT_Window:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self )
	self:SetText( "Daily Tasks" )
	self:SetSize( 250,360 )
--	self:SetBackColor( backColor )

	-- Position the window near the top and to left of center of the screen.
	local pos = DT_Settings.pos1 or 
				{ x=Turbine.UI.Display.GetWidth()/5, y=self:GetHeight()/2 }
	self:SetPosition( pos.x, pos.y )

	-- Create a label and add it to the window.
	self:AddField(Label, "Level:", {x=15,y=43}, {x=45,y=16} )

	-- Create a level text field and add it to the window.
	self.level = self:AddField(TextBox, player:GetLevel(), {x=57,y=46}, {x=24,y=15} )
	player.LevelChanged = function(sender, args)
		local lvl = player:GetLevel()
		DT_Settings.player[pname] = lvl
		if pname==self.playerButton:GetText() then
			self.level:SetText( lvl )
		end
	end

	-- Create a Player menu button
	self.playerButton = self:AddField(DropMenu, pname, {x=88,y=43}, {x=142,y=20} )
	local action = function(args)
		self.level:SetText(DT_Settings.player[args])
		pname = args
	end
	self.playerButton.Menu.Click = function()
		self.playerButton:BuildMenu(Player_list,action)
	end

	-- Create a label and add it to the window.
	self:AddField(Label, "Filter:", {x=15,y=69}, {x=50,y=16} )

	-- Create a Filter menu button
	self.filterButton = self:AddField(DropMenu, "All", {x=88,y=68}, {x=94,y=20} )
	local list = DT_Sort(DT_Filters)
	local action = function(args)
		if args=="Junk" then
			self.locationButton:SetText( All )
		end
	end
	self.filterButton.Menu.Click = function()
		self.filterButton:BuildMenu(list,action)
	end

	-- Rep-Only check box
	local box = self:AddField(CheckBox, "R-O", {x=190,y=70}, {x=45,y=16} )
	box.CheckedChanged = function( sender,args )
		RepOnly = sender:IsChecked() or nil
		print((RepOnly and "Enabled " or "Disabled ").."Rep-Only.")
	end
	Vinny.Common.ToolTip(box,2,-20,"Only show tasks that increase rep.",240)

	-- Create a label and add it to the window.
	self:AddField(Label, "Location:", {x=15,y=94}, {x=70,y=16} )

	-- Create a Location menu button
	self.locationButton = self:AddField(DropMenu, All, {x=88,y=93}, {x=149,y=20} )
	self.locationButton.Menu.Click = function( sender,args )
		local selected = self.locationButton:GetText()
		local ZoneMenu = Turbine.UI.ContextMenu()
		local menuItems,Item = ZoneMenu:GetItems()
		Item = Turbine.UI.MenuItem( All,true,All==selected )
		DT_Click(Item,self.locationButton)
		menuItems:Add( Item )
		for i,name in ipairs(Zone_list) do
			Item = Turbine.UI.MenuItem( name )
			local LocMenu = Item:GetItems() -- add sub-menu
			for j,loc in ipairs(DT_Sort(DT_Data.zone[name])) do
				local locItem = Turbine.UI.MenuItem(loc,true,loc==selected)
				DT_Click(locItem,self.locationButton)
				LocMenu:Add( locItem )
			end
			menuItems:Add( Item )
		end
		local x,y = self.locationButton:PointToScreen(-5,20)
		ZoneMenu:ShowMenuAt(x,y)
	end

	-- Check Nearest button
	self.checkButton = self:AddField(Button, "Check nearest", {x=15,y=122}, {x=112,y=20} )
	local slot = Turbine.UI.Lotro.Quickslot()
	slot:SetParent( self.checkButton )
    slot:SetPosition( 2,3 )
    slot:SetSize( 150,15 )
    slot:SetShortcut(Turbine.UI.Lotro.Shortcut( Alias, "/dtn ;loc" ))
    slot:SetAllowDrop( false )

	-- Homestead check box
	self.Homestead = self:AddField(CheckBox, "Homestead", {x=140,y=125}, {x=95,y=16} )
	self.Homestead.CheckedChanged = function( sender,args )
		Homestead = sender:IsChecked()
		print((Homestead and "Enabled " or "Disabled ").."Homestead.")
		if Homestead then
			self.locationButton:SetText( All )
			if sender:IsShiftKeyDown() then
				self.filterButton:SetText( "Complete" )
				box:SetChecked(true)
				DT_Command:Execute("dtc","")
			end
		end
	end
	Vinny.Common.ToolTip(self.Homestead,2,-20,"Show homestead board range.",205)

	-- Create a Zones button
	self.zonesButton = self:AddField(Button, "List Zones", {x=15,y=152}, {x=90,y=20} )
	self.zonesButton.Click = function( sender,args )
        DTZ_Command:Execute("dtz","")
	end

	-- Create a Locations button
	self.locsButton = self:AddField(Button, "List Locations", {x=115,y=152}, {x=120,y=20} )
	self.locsButton.Click = function( sender,args )
        DTL_Command:Execute("dtl","")
	end

	-- Create an Items button
	self.itemsButton = self:AddField(Button, "List Items", {x=15,y=180}, {x=90,y=20} )
	self.itemsButton.Click = function( sender,args )
		if sender and sender:IsShiftKeyDown() then
			DTP_Command:Execute("dtp","tc")
			return
		end
        local txt,lvl = self.level:GetText()
        if txt and txt~='' then
            lvl = tonumber(txt)
            if not lvl or lvl<1 or lvl>Max_lvl then
                printe("Invalid level.")
                return
            end
        end
        txt = self.locationButton:GetText()
		if txt==All then txt = "" end
		local cmd = DT_Filters[ self.filterButton:GetText() ]
        DT_Command:Execute(cmd,txt,lvl)
	end

	-- Create a Rep.window button
	self.rwButton = self:AddField(Button, "Rep window", {x=115,y=180}, {x=120,y=20} )
	self.rwButton.Click = function( sender,args )
		DT_Rwindow:SetVisible( true )
	end

	-- Create a Move list button
	self.moveButton = self:AddField(Button, "Move list to bag #", {x=35,y=207}, {x=140,y=20})
	self.moveButton.Click = function( sender,args )
        DTB_Command:Execute("dtb",self.bagButton:GetText())
	end

	-- Create a Bag menu button
	local pack = player:GetBackpack()
	local cnt = math.floor(pack:GetSize()/15+1)
	local bag = DT_Settings.bag and tostring(cnt-1) or "1"
	self.bagButton = self:AddField(DropMenu, bag, {x=180,y=207}, {x=40,y=20} )
	Bag_list[cnt] = nil -- truncate for fewer bags
	self.bagButton.Menu.Click = function()
		self.bagButton:BuildMenu(Bag_list)
	end

	-- Create a Vault Items button
	self.vaultButton = self:AddField(Button, "Vault Items", {x=15,y=234}, {x=100,y=20} )
	self.vaultButton.Click = function( sender,args )
		DT_Command:Execute("dtv","")
	end

	-- Create a Shared Storage Items button
	self.sharedButton = self:AddField(Button, "Shared Items", {x=125,y=234}, {x=110,y=20} )
	self.sharedButton.Click = function( sender,args )
		DT_Command:Execute("dts","")
	end

	-- Create a Duplicate Vault Items button
	self.dupButton = self:AddField(Button, "Dup. Vault", {x=15,y=260}, {x=100,y=20} )
	self.dupButton.Click = function( sender,args )
		DT_Command:Execute("dtd","v")
	end

	-- Create a Duplicate Shared Items button
	self.dupButton = self:AddField(Button, "Dup. Shared", {x=125,y=260}, {x=110,y=20} )
	self.dupButton.Click = function( sender,args )
		DT_Command:Execute("dtd","")
	end

	-- Create a label and add it to the window.
	self:AddField(Label, "Drop trophy items here:", {x=40,y=286}, {x=170,y=16} )

	-- Create a Slot
	self.Slot = self:AddField(Slot, nil, {x=110,y=305}, {x=Qsize,y=Qsize} )
	self.Slot:SetBackColor( Turbine.UI.Color( 0.2, 0.2, 0.2 ) )
	local Blank = self.Slot:GetShortcut()
	self.Slot.DragDrop = function( sender, args )
		local shortcut = sender:GetShortcut()
		local itemType = shortcut:GetType()
		if itemType==0 then return end
		local itemData = shortcut:GetData()
		if sender:IsAltKeyDown() then print("Type="..itemType..", Data="..itemData) end
		if itemType~=2 then sender:SetShortcut(Blank) return end
		local item = shortcut:GetItem()
		if item:GetCategory()==207 then
			local ro=RepOnly
			if RepOnly then RepOnly=false end
			all,dtx = true; DT_Item(item)
			RepOnly = ro
		else printe(item:GetName().." is not a trophy task item.") end
	end
end
DT_window = DT_Window()

-- Rep  windows
DT_RWindow = class( Turbine.UI.Lotro.Window )
DT_RKWindow = class( Turbine.UI.Lotro.Window )
DT_RIWindow = class( Turbine.UI.Lotro.Window )
local replist,RepMap = {Any}, {[Any]="++"}
for code,name in pairs(DT_Data.rep) do
	table.insert(replist,name)
	RepMap[name] = code
end
table.sort(replist)

function DT_RWindow:AddField(control, text, pos, size)
	local field = control()
	field:SetParent( self )
	if text then field:SetText( text ) end
	field:SetPosition( pos.x,pos.y )
	field:SetSize( size.x,size.y )
	if control==Button or control==DropMenu or not text then return field end
	field:SetFont( labelFont )
	field:SetForeColor( foreColor )
	field:SetBackColor( backColor )
	field:SetTextAlignment( Center )
	return field
end

function DT_RKWindow:AddBox(name,line,code)
	local box = CheckBox()
	box:SetParent( self )
	box:SetForeColor( foreColor )
	box:SetPosition( 30,20*line )
	box:SetSize( self:GetWidth()-40, 20 )
	box:SetTextAlignment( Left )
	box:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
	box:SetText( " "..name )
	box.name = name
	box.code = code
	box:SetChecked(DT_rep[code])
	box.CheckedChanged = function( sender,args )
		local v = sender:IsChecked()
		local str = v and "Set" or "Cleared"
		DT_rep[sender.code] = v or nil
		print(str.." Kindred rep for "..sender.name )
	end
end

function DT_RKWindow:Constructor(list)
	Turbine.UI.Lotro.Window.Constructor( self )
	self:SetText( "Kindred Rep" )

	-- Position the window near the top and to left of center of the screen.
	pos = { x=Turbine.UI.Display.GetWidth()/3+280, y=self:GetHeight()/3 }
	self:SetPosition( pos.x, pos.y )
	self:SetSize( 255,40+20*#replist )

	-- Create a checkbox and add it to the window.
	for ix,name in ipairs(replist) do
		if name~=Any and name~="<none>" then
			self:AddBox(name,ix,RepMap[name])
		end
	end
end

function DT_RIWindow:AddBox(name,line,code)
	local box = CheckBox()
	box:SetParent( self )
	box:SetForeColor( foreColor )
	box:SetPosition( 30,20*line )
	box:SetSize( self:GetWidth()-40, 20 )
	box:SetTextAlignment( Left )
	box:SetFont( Turbine.UI.Lotro.Font.TrajanPro16 )
	box:SetText( " "..name )
	box.name = name
	box.code = code
	box:SetChecked(DT_rep[code] ~= nil)
 	box:SetEnabled(not DT_rep[code])
	box.CheckedChanged = function( sender,args )
		local v = sender:IsChecked()
		local str = v and "Set" or "Cleared"
		if v then DT_rep[sender.code] = false
		else DT_rep[sender.code] = nil end
		print(str.." Ignore rep for "..sender.name )
	end
end

function DT_RIWindow:Constructor(list)
	Turbine.UI.Lotro.Window.Constructor( self )
	self:SetText( "Ignore Rep" )

	-- Position the window near the top and to left of center of the screen.
	pos = { x=Turbine.UI.Display.GetWidth()/3+280, y=self:GetHeight()/3 }
	self:SetPosition( pos.x, pos.y )
	self:SetSize( 255,40+20*#replist )

	-- Create a checkbox and add it to the window.
	for ix,name in ipairs(replist) do
		if name~=Any and name~="<none>" then
			self:AddBox(name,ix,RepMap[name])
		end
	end
end

function DT_RWindow:Constructor(list)
	Turbine.UI.Lotro.Window.Constructor( self )
	self:SetText( "DT Reputation" )
	self:SetSize( 275,270 )

	-- Position the window near the top and to left of center of the screen.
	local pos = DT_Settings.pos2 or 
				{ x=Turbine.UI.Display.GetWidth()/4+5, y=self:GetHeight()/3 }
	self:SetPosition( pos.x, pos.y )

	-- Create a label and add it to the window.
	self:AddField(Label, "Rep:", {x=15,y=43}, {x=35,y=16} )

	-- Create a Rep Group menu button
	self.repButton = self:AddField(DropMenu, Any, {x=54,y=43}, {x=195,y=20} )
	self.rep = RepMap[Any]
	local action = function(args)
		self.rep = RepMap[args]
	end
	self.repButton.Menu.Click = function()
		self.repButton:BuildMenu(replist,action)
	end

	-- Create a Rep.locations button
	self.locsButton = self:AddField(Button, "List rep locations", {x=75,y=70}, {x=140,y=20} )
	self.locsButton.Click = function( sender,args )
		if self.rep=="++" then printe("Select a rep group.")
		else DTL_Command:Execute("dtl",self.rep) end
	end

	-- Create a Rep.items button
	self.itemsButton = self:AddField(Button, "List rep items", {x=85,y=95}, {x=130,y=20} )
	self.itemsButton.Click = function( sender,args )
		local cmd = DT_Filters[ DT_window.filterButton:GetText() ]
		if self.rep=="++" then printe("Select a rep group.")
		else DT_Command:Execute(cmd,self.rep) end
	end

	-- Create a find Rep.uses button
	self.itemsButton = self:AddField(Button, "Find top rep item uses", {x=70,y=125}, {x=160,y=20} )
	self.itemsButton.Click = function( sender,args )
        DTR_Command:Execute("dtr","top")
	end

	-- Create a Rep.codes button
	self.codesButton = self:AddField(Button, "List rep codes", {x=85,y=155}, {x=130,y=20} )
	self.codesButton.Click = function( sender,args )
        DTR_Command:Execute("dtr","")
	end

	-- Create a List Kindred button
	self.rsButton = self:AddField(Button, "List Kindred Rep", {x=85,y=180}, {x=130,y=20} )
	Vinny.Common.ToolTip(self.rsButton,5,-20,"Shift for any rep.",180)
	self.rsButton.Click = function( sender,args )
        DTR_Command:Execute("dtr","list")
	end

	-- Create a Set Kindred button
	self.rsButton = self:AddField(Button, "Set Kindred Rep", {x=85,y=205}, {x=130,y=20} )
	self.rsButton.Click = function( sender,args )
		DT_RKwindow:SetVisible( true )
	end

	-- Create a Set Ignore button
	self.rsButton = self:AddField(Button, "Set Ignore Rep", {x=85,y=230}, {x=130,y=20} )
	self.rsButton.Click = function( sender,args )
		DT_RIwindow:SetVisible( true )
	end

end

DT_Rwindow = DT_RWindow()
DT_RKwindow = DT_RKWindow()
DT_RIwindow = DT_RIWindow()

-- Set Escape action
DT_window:SetWantsKeyEvents( true )
DT_window.KeyDown = function(sender, args)
	if( args.Action == Turbine.UI.Lotro.Action.Escape ) then
		DT_window:SetVisible( false )
		DT_Rwindow:SetVisible( false )
		DT_RKwindow:SetVisible( false )
		DT_RIwindow:SetVisible( false )
	end
end

DTW_Command = Turbine.ShellCommand()

function DTW_Command:Execute( command,args )
	if args=="?" then
		Vinny.Common.Help(help,"dtw")
		return
	end
	if args=="rep" then
		DT_Rwindow:SetVisible( true )
		return
	end
	if args=="kr" then
		DT_RKwindow:SetVisible( true )
		return
	end
	if args=="ir" then
		DT_RIwindow:SetVisible( true )
		return
	end
	DT_Settings.player[pname] = player:GetLevel()
    DT_window:SetVisible( true )
end

Turbine.Shell.AddCommand( "dtw",DTW_Command )

-- Check nearest board
local Coord = "^(%d+%.%d[NnSs]),(%d+%.%d[EeWw])$"
local Zloc = "^(.+): .+: (%d+%.%d[NS]), (%d+%.%d[EW])$"

DTN_Command = Turbine.ShellCommand()

function DTN_Command:Execute( command,args )
	if args=="" then print("Error: Missing location.") return end
	local reg,y,x = args:match(Zloc)
	if y then
		local r = Region[reg]
		local d1,y1,x1,ln = 999,locV(y,"Ss"), locV(x,"Ww"), ""
		local y2,x2,d
		for loc,t in pairs(DT_Data.loc) do
			if t.loc and t.r==r then
				local y0,x0 = t.loc:match(Coord)
				if not y then printe("Bad Loc for "..loc) return end
				y2,x2 = locV(y0,"Ss"), locV(x0,"Ww")
				d = distance(y1-y2,x1-x2)
				if d<d1 then d1 = d; ln = loc end
			end
		end
		d = string.format(" (%.1f units away).",d1)
		print("The nearest board to "..y..","..x.." is "..ln..d)
		DT_window.filterButton:SetText( "Complete" )
		DT_window.locationButton:SetText( ln )
		DT_window.Homestead:SetChecked(false)
		DT_window.itemsButton.Click()
	else print("Error: No location data in instance.") end
end

Turbine.Shell.AddCommand( "dtn",DTN_Command )
